==> 0.1.3.tar.gz: lib/src/model/field_editor.dart <==
part of fields.model;

class FieldEditorResources {
  String get style =>
"""
input {
  background-color:transparent;
  border-style:none;
  color:${fieldsTheme.colors.text};
}
input[type="text"] {  
  width:100%;
}
input[type="text"][disabled] { 
  color:${fieldsTheme.colors.textDisabled};
}
input[class=error] {
  border: 2px solid ${fieldsTheme.colors.error};
}
""";
  const FieldEditorResources();
}

abstract class FieldEditor<T> {
  
  static final _fieldEditorClassMirror = reflectClass(FieldEditor);
  static final _editors = new Map<Type, ClassMirror>();
  static ClassMirror _default;
    
  static bool isRegistered(Type fieldType) => _editors.containsKey(fieldType);
  static bool get isDefaultRegistered => _default != null;
  
  static void register(Type fieldType, Type editorType) {
    if (editorType == null) {
      throw new ArgumentError('editorType must be !null');
    }
    if (isRegistered(fieldType)) {
      throw new StateError('$fieldType already has a registered editorType');
    }
    final editorClassMirror = reflectClass(editorType);
    if (!_isAssignableFrom(_fieldEditorClassMirror, editorClassMirror)) {
      throw new ArgumentError(
          '$editorType must extend or implement $FieldEditor');
    }
    _editors[fieldType] = editorClassMirror;
  }
  
  static void registerDefault(Type editorType) {
    if (editorType == null) {
      throw new ArgumentError('editorType must be !null');
    }
    if (isDefaultRegistered) {
      throw new StateError('Default editorType is already registered.');
    }
    final editorClassMirror = reflectClass(editorType);
    if (!_isAssignableFrom(_fieldEditorClassMirror, editorClassMirror)) {
      throw new ArgumentError(
          '$editorType must extend or implement $FieldEditor');
    }
    _default = editorClassMirror;
  }
  
  static bool unregister(Type fieldType) => _editors.remove(fieldType) != null;
  
  static void unregisterDefault() {
    _default = null;
  }
  
  static void unregisterAll() {
    unregisterDefault();
    _editors.clear();
  }
    
  /*Element*/ get element;
  
  final Field field;
  final FieldEditorResources resources;
  
  final Type valueType = T;
  
  bool _isEnabled = true;
  bool 
    get isEnabled => _isEnabled;
    set isEnabled(bool isEnabled) {
      if (_isEnabled != isEnabled) {
        _isEnabled = isEnabled;
        render();
      }
    }
    
  bool get isValueReadOnly =>
      field.isReadOnly || 
      !isEnabled ||
      ( field.transform == null && !canTransformField(valueType, field.type) );
  
  get value {
    final value = field.value;
    if (field.transform != null) {
      return field.transform(value);
    } else if (canTransformField(field.type, valueType)) {
      return transformField(value, field.type, valueType);
    } else {
      return value.toString();
    }
  }
  
  set value(value) {
    if(isValueReadOnly) {
      throw new UnsupportedError('Cannot set a read-only value');
    }
    if (field.transform != null && field.transform.hasInverse) {
      field.value = field.transform.inverse(value);
    } else {
      field.value = transformField(value, valueType, field.type);
    }
  }
  
  factory FieldEditor.forField(Field field) {
    ClassMirror mirror = _default;
    if (field.editor != null) {
      mirror = field.editor;
    } else if (isRegistered(field.type)) {
      mirror = _editors[field.type];
    }
    if (mirror == null) {
      throw new StateError('No editor registered for type <${field.type}>');
    }
    return mirror.newInstance(const Symbol(''),[field]).reflectee;
  }
  
  FieldEditor(this.field, [this.resources = const FieldEditorResources()]) {
    field.values.listen((_) => render());
  }

  void render();
}

==> 0.2.3.tar.gz: lib/src/model/field_editor.dart <==
part of fields.model;

abstract class FieldEditor<T> {
  
  static final _fieldEditorClassMirror = reflectClass(FieldEditor);
  static final _editors = new Map<Type, ClassMirror>();
  static ClassMirror _default;
    
  static bool isRegistered(Type fieldType) => _editors.containsKey(fieldType);
  static bool get isDefaultRegistered => _default != null;
  
  static void register(Type fieldType, Type editorType) {
    if (editorType == null) {
      throw new ArgumentError('editorType must be !null');
    }
    if (isRegistered(fieldType)) {
      throw new StateError('$fieldType already has a registered editorType');
    }
    final editorClassMirror = reflectClass(editorType);
    if (!_isAssignableFrom(_fieldEditorClassMirror, editorClassMirror)) {
      throw new ArgumentError(
          '$editorType must extend or implement $FieldEditor');
    }
    _editors[fieldType] = editorClassMirror;
  }
  
  static void registerDefault(Type editorType) {
    if (editorType == null) {
      throw new ArgumentError('editorType must be !null');
    }
    if (isDefaultRegistered) {
      throw new StateError('Default editorType is already registered.');
    }
    final editorClassMirror = reflectClass(editorType);
    if (!_isAssignableFrom(_fieldEditorClassMirror, editorClassMirror)) {
      throw new ArgumentError(
          '$editorType must extend or implement $FieldEditor');
    }
    _default = editorClassMirror;
  }
  
  static bool unregister(Type fieldType) => _editors.remove(fieldType) != null;
  
  static void unregisterDefault() {
    _default = null;
  }
  
  static void unregisterAll() {
    unregisterDefault();
    _editors.clear();
  }
    
  /*Element*/ get element;
  
  final Field field;
  
  final Type valueType = T;
  
  bool _isEnabled = true;
  bool 
    get isEnabled => _isEnabled;
    set isEnabled(bool isEnabled) {
      if (_isEnabled != isEnabled) {
        _isEnabled = isEnabled;
        render();
      }
    }
    
  bool get isValueReadOnly =>
      field.isReadOnly || 
      !isEnabled ||
      ( field.transform == null && 
      !FieldTransform.canTransform(valueType, field.type) );
  
  get value {
    final value = field.value;
    if (field.transform != null) {
      return field.transform(value);
    } else if (FieldTransform.canTransform(field.type, valueType)) {
      return FieldTransform.transform(value, field.type, valueType);
    } else {
      return value.toString();
    }
  }
  
  set value(value) {
    if(isValueReadOnly) {
      throw new UnsupportedError('Cannot set a read-only value');
    }
    if (field.transform != null && field.transform.hasInverse) {
      field.value = field.transform.inverse(value);
    } else {
      field.value = FieldTransform.transform(value, valueType, field.type);
    }
  }
  
  factory FieldEditor.forField(Field field) {
    ClassMirror mirror = _default;
    if (field.editor != null) {
      mirror = field.editor;
    } else if (isRegistered(field.type)) {
      mirror = _editors[field.type];
    }
    if (mirror == null) {
      throw new StateError('No editor registered for type <${field.type}>');
    }
    return mirror.newInstance(const Symbol(''),[field]).reflectee;
  }
  
  FieldEditor(this.field) {
    field.values.listen((_) => render());
  }

  void render();
}
